unit gui;
{$IFDEF FPC}{$mode objfpc}{$H+}{$ENDIF}

interface

uses
{$IFDEF FPC}LResources,{$ENDIF}
{$IFNDEF UNIX}  Windows,ShellAPI,ShlObj, {$ENDIF}
Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ToolWin, ComCtrls, ExtCtrls,IniFiles,
    define_types,sortdicom,dicom,parconvert,filename, convert,userdir,  paramstrs,
  dicomtypes,nii_crop,nii_orient, nii_4dto3d,nii_asl,nii_reslice, Menus,nii_3dto4d,prefs;
type
   TMainForm = class(TForm)
    Label1: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Edit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Copy1: TMenuItem;
    DICOMtoNIfTI1: TMenuItem;                                              
    AnonymizeDICOM1: TMenuItem;
    Exit1: TMenuItem;
    NIfTI3D4D1: TMenuItem;
    ModifyNIfTI1: TMenuItem;
    Preferences1: TMenuItem;
    Memo1: TMemo;
    OpenHdrDlg: TOpenDialog;
    Panel1: TPanel;
    TypeCombo: TComboBox;
    ResliceNIfTI1: TMenuItem;
    Deletenondcm1: TMenuItem;

    procedure FormClose(Sender: TObject; var vAction: TCloseAction);
    procedure FormDropFiles(Sender: TObject; const FileNames: array of String);
    function OpenDialogExecute (lCaption: string;lAllowMultiSelect,lForceMultiSelect: boolean; lFilter: string): boolean;//; lAllowMultiSelect: boolean): boolean;
    procedure CheckPrefs (var lPrefs: TPrefs; lWrite: boolean);
    function ConvertDCM2NII (lFilename: string; lPrefs: TPrefs): boolean;
    procedure dcm2niiBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Preferences1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TypeComboChange(Sender: TObject);
    procedure AnonymizeDICOM1Click(Sender: TObject);
    procedure ModifyNIfTI1Click(Sender: TObject);
    procedure NIfTI3D4D1Click(Sender: TObject);
    procedure ResliceNIfTI1Click(Sender: TObject);
    procedure Deletenondcm1Click(Sender: TObject);
  private

  {$IFNDEF FPC}//if Delphi
            procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
  {$ENDIF}
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses untar,pref_form, nifti_form,niftiutil{$IFNDEF UNIX},ActiveX {$ENDIF};
{$IFNDEF FPC}
{$R *.DFM}
{$R windowsxp.res}
{$ENDIF}



procedure MsgX (lStr: string);
begin
         MainForm.Memo1.Lines.Add(lStr);
end;

procedure SetOutputFormat (lItemIndex: integer; var lPrefs: TPrefs);
begin
    //next: options for reading;
    lPrefs.SPM2 := false;
    lPrefs.fourD := true;
    lPrefs.SingleNIIFile := true;
    lPrefs.GZip := false;
    case lItemIndex of
         0: begin//spm2
                lPrefs.SPM2 := true;
                lPrefs.fourD := false;
                lPrefs.SingleNIIFile := false;
                end;
         1: begin//spm5
                lPrefs.fourD := false;
                lPrefs.SingleNIIFile := false;
                end;
         2: lPrefs.SingleNIIFile := false;//4D Hdr/Img
         4: lPrefs.GZip := true;//FSL compressed
    end;//case

end;

function is4D (var lHdr: TNIFTIhdr): boolean;
begin
     if lHdr.dim[4] > 1 then
        result := true
     else
         result := false;
end;
function SelectProcessNIFTI (var lHdr: TNIFTIhdr; var lFilename: string): integer;
begin
     result := -1; //returns -1 if error
     if is4D(lHdr) then begin
        NIfTIForm.Combo3D.visible := false;
        NIfTIForm.Combo4D.visible := true;
     end else begin
         //NIfTIForm.Combo3D.itemIndex := 2;
         NIfTIForm.Combo3D.visible := true;
         NIfTIForm.Combo4D.visible := false;
     end;
     NiftiForm.Combo4DChange(nil);
     NIftiForm.caption := extractfilename(lFilename);
     //next - let user specify task
     NiftiForm.showmodal;
     if (NiftiForm.ModalResult = mrCancel) then
        exit;
     if is4D(lHdr) then
        result := NiftiForm.Combo4D.ItemIndex
     else
         result := NiftiForm.Combo3D.ItemIndex;

end;
//procedure ProcessNIfTI(lInFilename: string; lPrefs: TPrefs);
procedure ProcessNIfTI(lFilenames : TStrings; lPrefs: TPrefs);
var
   l4D, lPrev4D,lByteSwap: boolean;
   lINc,lProcess: integer;
   lExt,lFilename,lOutname: string;
   lHdr: TNIFTIhdr;
begin
     if lFilenames.Count < 1 then exit;
     for lInc := 1 to lFilenames.Count do begin
         lFilename := lFilenames.Strings[lInc-1];
         lExt := UpCaseExt(lFilename);
         if lExt ='.IMG' then
            lFilename :=  changefileext(lFilename,'.hdr');
         if not NIFTIhdr_LoadHdr (lFilename, lHdr, lByteSwap) then begin
            MsgX('Unable to read as NifTI/Analyze' + lFilename);
            exit;
         end;
         lProcess := 0; //always set in if statement - set only to avoid compiler warning
         lPrev4D := false; //ignored in if statement - set only to avoid compiler warning
         l4D := is4D(lHdr);
         //choose process
         if (lInc = 1) or (l4D <> lPrev4D) then begin
            lProcess := SelectProcessNIFTI(lHdr,lFilename);
            if lProcess < 0 then exit;
            lPrev4D := l4D;
         end;
         //next - convert image as specified
         SetOutputFormat(NIfTIForm.TypeCombo.ItemIndex,lPrefs);
         if l4D then begin
         case lProcess of
              0: ChangeNIfTISubformat(lFilename,lHdr,lPrefs);
              1: Reorder4D(lFilename, lHdr, lByteSwap,lPrefs);
              2: Clip4D(lFilename, lHdr, false,lPrefs,NiftiForm.StartEdit.value, NiftiForm.EndEdit.value);
              3: Float32NIfTI(lFilename, lPrefs);
               4: FormulaNIfTI(lFilename,lPrefs, NiftiForm.ScaleEdit.value, NiftiForm.PowerEdit.value);
              5: ASL_subtract(lFilename,false,{subtract} (NiftiForm.AsLCombo.itemIndex ),lPrefs);
         end; //case combo
     end else begin //if 4d else 3d
         case lProcess of
              0: ChangeNIfTISubformat(lFilename,lHdr,lPrefs);
              1: Reorient(lFilename,lHdr, lPrefs,false,false);
              2: begin
                 lOutname := Reorient(lFilename,lHdr, lPrefs,false,false);
                 if lOutname <> '' then
                    CropNIfTI(lOutname,lPrefs);
                 end;//2
         end; //case 3d
     end; //if 4d else 3d end
 end; //for each image
end;

function TMainForm.ConvertDCM2NII (lFilename: string; lPrefs: TPrefs): boolean;
//returns true if files treated as DICOM or PAR/REC - these will search entire folder
var
   lOutDir,lExt: String;
   lStartTime: DWord;
   lStrings : TStrings;
begin
     DecimalSeparator := '.';
     result := false;
     if (not Fileexists(lFilename)) and (not DirExists(lFilename)) then
        exit;
     result := true;
     Memo1.lines.clear;
     MsgX(kVers);
     refresh;
     Memo1.lines.add('Converting '+lFilename);
     lOutDir := extractfiledir(lFilename);
     //lOutDir := 'C:\ct_musc';xxx
     {$IFNDEF UNIX}lStartTime := GetTickCount;{$ENDIF}
     if DirExists(lFilename) then begin
        RecursiveFolderSearch(lFilename,lFilename,lPrefs,0);
        lPrefs.NameAppend := '';
     end else begin
          lExt := UpCaseExt(lFilename);
          if (lExt = '.REC') or (lExt = '.PAR') then
             LoadFileListPARREC(lFilename,lOutDir,lPrefs)
          else if (lExt = '.TGZ')  then
                DeTGZ(lFilename,lPrefs)
          else if (IsNiftiExt (lFilename)) or (IsVOIExt (lFilename)) then begin
              result := false;
              lStrings := TStringList.Create;
              lStrings.add(lFilename);
              ProcessNIfTI(lStrings,lPrefs);
              lStrings.Free;
          end else begin
              if DirExists(lOutDir) then
                 RecursiveFolderSearch(lOutDir,lOutDir,lPrefs,0)
              else
                  LoadFileList(lFilename,lOutDir,lPrefs);
              lPrefs.NameAppend := '';
          end;
     end;
     {$IFNDEF UNIX}Memo1.lines.add('Conversion completed in '+inttostr(GetTickCount-lStartTime)+' ms');
     {$ELSE}
     Memo1.lines.add('Conversion completed');
      {$ENDIF}
end;

{$IFNDEF FPC}//if delphi
procedure TMainForm.WMDropFiles(var Msg: TWMDropFiles);
var
  CFileName: array[0..MAX_PATH] of Char;
  lFilename: string;
  lDone: boolean;
  lInc: integer;
  lPrefs: TPrefs;
begin
     CheckPrefs(lPrefs,False);
  lDone := false;
  lInc := 0;                                                         
  try
   while (DragQueryFile(Msg.Drop, lInc, CFileName, MAX_PATH) > 0) and (not lDone) do begin
         lFilename := CFilename;
         lDone := ConvertDCM2NII( lFilename,lPrefs);
         Msg.Result := 0;
         inc(lInc);
    end; //while
  finally
    DragFinish(Msg.Drop);
  end;
end;

function BrowseDialog
 (const Title: string): string;
var
  iFlag: integer;
  lpItemID : PItemIDList;
  BrowseInfo : TBrowseInfo;
  DisplayName : array[0..MAX_PATH] of char;
  TempPath : array[0..MAX_PATH] of char;
begin
  iFlag :=  BIF_RETURNONLYFSDIRS;
    //iFlag :=  BIF_BROWSEINCLUDEFILES;
    //iFlag :=  BIF_BROWSEFORCOMPUTER;
    //iFlag :=  BIF_BROWSEFORPRINTER;
  Result:='';
  FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
  with BrowseInfo do begin
	hwndOwner := Application.Handle;
	pszDisplayName := @DisplayName;
	lpszTitle := PChar(Title);
	ulFlags := iFlag;
  end;
  lpItemID := SHBrowseForFolder(BrowseInfo);
  if lpItemId <> nil then begin
	SHGetPathFromIDList(lpItemID, TempPath);
	Result := TempPath;
	GlobalFreePtr(lpItemID);
  end;     
end;
{$ELSE}
function BrowseDialog
 (const Title: string): string;
 var
 lSelectDirectoryDialog1: TSelectDirectoryDialog;
begin
    result := '';
    lSelectDirectoryDialog1.Create(MainForm);
    lSelectDirectoryDialog1.title := Title;
    if not lSelectDirectoryDialog1.execute then exit;
    result := lSelectDirectoryDialog1.Filename;
    lSelectDirectoryDialog1.free;
end;
{$ENDIF}
procedure TMainForm.dcm2niiBtnClick(Sender: TObject);
var
   sTitle,lDirName: string;
   //iFlag: integer;
   lPrefs: TPrefs;
begin
        CheckPrefs(lPrefs,False);
    {$IFNDEF UNIX}
  sTitle:='Choose a folder with DICOM images';
      lDirName := BrowseDialog(sTitle);
      {$ELSE}
      if not OpenDialogExecute('Select DICOM images you wish to convert)',true,false,kAnyFilter) then
         exit;

         lDirName := extractfiledir( OpenHdrDlg.Filename);
      {$ENDIF}

      ConvertDCM2NII(lDirName,lPrefs);
end;



procedure TMainForm.CheckPrefs (var lPrefs: TPrefs; lWrite: boolean);
begin
    if lWrite then begin
       //showmessage('w');
       //options if writing
       if lPrefs.SPM2 then
          TypeCombo.ItemIndex := 0
       else if not lPrefs.FourD  then
            TypeCombo.ItemIndex := 1
       else if not lPrefs.SingleNIIFile then
            TypeCombo.ItemIndex := 2
       else if lPrefs.GZip then
            TypeCombo.ItemIndex := 4
       else
           TypeCombo.ItemIndex := 3; //FSL uncompressed
       exit;
    end;
  SetDefaultPrefs (lPrefs);
  PrefsForm.ReadPrefs(lPrefs);
  SetOutputFormat(TypeCombo.ItemIndex,lPrefs);
   // showmessage('read'+inttostr(TypeCombo.ItemIndex));


  lPrefs.AnonymizeSourceDICOM := false;
end;

(*procedure Fz;
var
   lPrefs: TPrefs;
   lByteSwap: boolean;
   lExt,lFilename,lOutname,lNameWOExt: string;
   lHdr: TNIFTIhdr;
begin
  lFilename := 'C:\dti64\rapid\fz3.nii';
  lFilename := 'C:\t1\mx.nii';
     if not NIFTIhdr_LoadHdr (lFilename, lHdr, lByteSwap) then begin
        MsgX('Unable to read as NifTI/Analyze' + lFilename);
        exit;
     end;
     MainForm.CheckPrefs(lPrefs,False);
  Reorient(lFilename, lHdr,lPrefs,false);
end;     *)

(*procedure Fz;
var
   lPrefs: TPrefs;
   lF: string;
begin
     lF := 'C:\iceland\temp';
      SetDefaultPrefs (lPrefs);
       lPrefs.AnonymizeSourceDICOM := true;
     MainForm.ConvertDCM2NII(lF,lPrefs);
end;*)

procedure TMainForm.FormCreate(Sender: TObject);
begin
    {$IFDEF Darwin}
      Exit1.visible := false;
    {$ENDIF}
      {$IFNDEF UNIX}DragAcceptFiles(Handle, True);{$ENDIF}
    //Memo1.Lines.Add(kVers);
    DecimalSeparator := '.';
    Application.HintHidePause := 30000;
    //fz;
end;          



procedure TMainForm.Exit1Click(Sender: TObject);
begin
     Close;
end;

procedure TMainForm.Copy1Click(Sender: TObject);
begin
	Memo1.SelectAll;
	Memo1.CopyToClipboard;
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
     Showmessage(kVers);
end;

procedure TMainForm.Preferences1Click(Sender: TObject);
var
   lPrefs: TPrefs;
begin
     PrefsForm.ReadPrefs(lPrefs);
     PrefsForm.Showmodal;
     if (PrefsForm.ModalResult = mrCancel) then
        PrefsForm.WritePrefs(lPrefs);;
end;

procedure TMainForm.FormShow(Sender: TObject);
var
   lPrefs: TPrefs;
   lIniName: string;
begin
     SetDefaultPrefs(lPrefs);
    lIniName := IniName;//changefileext(paramstr(0),'.ini');
     {$IFNDEF UNIX}
     if (ParamCount > 0) then
        ProcessParamStrs
     else
         IniFile(True,lIniName, lPrefs);
     {$ELSE}
     IniFile(True,lIniName, lPrefs);
     {$ENDIF}
    CheckPrefs(lPrefs,True);
     PrefsForm.WritePrefs(lPrefs);
     NIfTIForm.TypeCombo.ItemIndex := TypeCombo.ItemIndex;
     MsgX(kVers);
     //ConvertDCM2NII('C:\tq\SerieMR-0029-0005.dcm',lPrefs);
     //lIniName := 'C:\swap\9_mm_FLAIR.nii';
     //SuperReorient(lIniName,lPrefs);
      //ConvertDCM2NII('C:\tq\IM_0001.dcm',lPrefs);
end;

procedure TMainForm.TypeComboChange(Sender: TObject);
begin
     NIfTIForm.TypeCombo.ItemIndex := TypeCombo.ItemIndex;
end;

procedure TMainForm.AnonymizeDICOM1Click(Sender: TObject);
var
   sTitle,lDirName: string;
   lPrefs: TPrefs;
begin
  CheckPrefs(lPrefs,False);
  lPrefs.AnonymizeSourceDICOM := true;
  sTitle:='Choose a folder with DICOM images';
    lDirName := BrowseDialog(sTitle);
    ConvertDCM2NII(lDirName,lPrefs);
end;

function TMainForm.OpenDialogExecute (lCaption: string;lAllowMultiSelect,lForceMultiSelect: boolean; lFilter: string): boolean;//; lAllowMultiSelect: boolean): boolean;
var
   lNumberofFiles: integer;
begin
	OpenHdrDlg.Filter := lFilter;//kAnaHdrFilter;//lFilter;
	OpenHdrDlg.FilterIndex := 1;
	OpenHdrDlg.Title := lCaption;
	if lAllowMultiSelect then
		OpenHdrDlg.Options := [ofAllowMultiSelect,ofFileMustExist]
	else
		OpenHdrDlg.Options := [ofFileMustExist];
	result := OpenHdrDlg.Execute;
	if not result then exit;
	if lForceMultiSelect then begin
		lNumberofFiles:= OpenHdrDlg.Files.Count;
		if  lNumberofFiles < 2 then begin
			Showmessage('Error: This function is designed to overlay MULTIPLE images. You selected less than two images.');
			result := false;
		end;
	end;
end;

procedure TMainForm.FormClose(Sender: TObject; var vAction: TCloseAction);
var
   lPrefs: TPrefs;
   lIniName: string;
begin
    lIniName := IniName;//changefileext(paramstr(0),'.ini');
    CheckPrefs(lPrefs,False);
    IniFile(False,lIniName, lPrefs);
end;

procedure TMainForm.FormDropFiles(Sender: TObject;
  const FileNames: array of String);
var
   lI,lN: integer;
  lPrefs: TPrefs;
  lDone: boolean;
  lFilename: string;
begin
     lDone := false;
     CheckPrefs(lPrefs,False);
     lN := length(FileNames);
     if lN < 1 then
        exit;
      lI := 0;
      while (lI < lN) and (not lDone) do begin
         lFilename := Filenames[lI];
         lDone := ConvertDCM2NII( lFilename,lPrefs);
         inc(lI);
     end;
end;

procedure TMainForm.ModifyNIfTI1Click(Sender: TObject);
var
   lPrefs: TPrefs;
   //lPos: integer;
begin
 if not OpenDialogExecute('Select NIfTI images you wish to modify)',true,false,kImgFilter) then
    exit;
   CheckPrefs(lPrefs,False);

 ProcessNIfTI(OpenHdrDlg.Files,lPrefs);
end; //ModifyNIfTI1Click

procedure TMainForm.NIfTI3D4D1Click(Sender: TObject);
var lStrings: TStringList;
   lPrefs: TPrefs;
begin
 if not OpenDialogExecute('Select the 3D NIfTI images you wish to stack)',true,false,kImgFilter) then
    exit;
 lStrings := TStringList.Create;
 lStrings.addstrings(OpenHdrDlg.Files);
 CheckPrefs(lPrefs,False);
 Stack3Dto4D(lStrings, False, lPrefs);
 lStrings.Free;
end;

procedure TMainForm.ResliceNIfTI1Click(Sender: TObject);
var
   lDestName,lSourceName,lTargetName: string;
   lPos: integer;
   lPrefs: TPrefs;
begin
         CheckPrefs(lPrefs,False);
     Memo1.lines.clear;
     refresh;
     MsgX(kVers);
     MsgX('This function reslices source images to match the dimensions of a target image.');
     MsgX(' Images are assumed to be coregistered.');
     MsgX(' The resulting images will have the orientation, voxel size and bounding box of the target image.');
     MsgX(' Resliced images will be given the prefix ''r''.');
     MsgX(' This function uses trilinear interpolation - there may be some loss of precision.');
     if not OpenDialogExecute('Select target image',true,false,kImgFilter) then
        exit;
     lTargetName := OpenHdrDlg.Filename;
     if not OpenDialogExecute('Select images you wish to reslice to match target)',true,false,kImgFilter) then
        exit;
     for lPos := 1 to OpenHdrDlg.Files.Count do begin
         lSourceName := OpenHdrDlg.Files[lPos-1];
         lDestName := ChangeFilePrefix (lSourceName,'r');
         MsgX('Reslicing '+lSourceName +' to match dimensions of '+lTargetname+' resliced image = '+lDestName);
         Reslice2Targ (lSourceName,lTargetName,lDestName, lPrefs );

     end;

end;


procedure DelRecursiveFolderSearch (lFolderName: string; lMaxDepth, lDepth: integer);
var
 lNewDir,lNewName,lFilename,lExt: String;
 lSearchRec: TSearchRec;
begin
 lNewDir := lFolderName+PathDelim;
{$IFDEF UNIX}
 if FindFirst(lNewDir+'*',faAnyFile-faSysFile,lSearchRec) = 0 then begin
{$ELSE}
 if FindFirst(lNewDir+'*.*',faAnyFile-faSysFile,lSearchRec) = 0 then begin
{$ENDIF}
    lFilename := '';
    repeat
      lNewName := lNewDir+lSearchRec.Name;
      if  (lSearchRec.Name <> '.') and (lSearchRec.Name <> '..') then begin
            if DirExists(lNewName) then begin
               if lDepth < lMaxDepth then begin
                  DelRecursiveFolderSearch(lNewName,lMaxDepth,lDepth+1);
               end;
               //exit;//4/4/2008
            end else
                lFilename := lNewname;
      end;
      if (lFilename <> '') and (not DirExists(lNewName)) then begin
	      	lExt := UpCaseExt(lFilename);
                if (lExt <> '.DCM') then begin
                   msgx('del '+lFilename);
                   DeleteFile(lFilename);
                end;
      end;
    until (FindNext(lSearchRec) <> 0);

 end;
 FindClose(lSearchRec);
end;


procedure TMainForm.Deletenondcm1Click(Sender: TObject);
var
   sTitle,lDirName: string;
begin
  Showmessage('Warning: this command will delete all files that do not have the extension .dcm') ;
  sTitle:='Choose a folder with DICOM images';
    lDirName := BrowseDialog(sTitle);
    DelRecursiveFolderSearch(lDirName,32,1);
end;



{$IFDEF UNIX}



initialization
  {$I gui.lrs}
{$ELSE} //not unix: windows
initialization
{$IFDEF FPC}
  {$I gui.lrs}
 {$ENDIF}
  OleInitialize(nil);

finalization
  OleUninitialize
{$ENDIF}
end.


