unit prefs;
{$H+}
interface
uses
  inifiles, define_types,SysUtils;

type
  TPrefs = record
         RecursiveUseNameAppend,AnonymizeSourceDICOM, ManualNIfTIConv,Anonymize,
          SingleNIIFile,Gzip,SPM2,enablereorient,createoutputfolder,
         AppendDate,AppendAcqSeries,AppendProtocolName,AppendPatientName,AppendFilename,
         everyfile,fourD,Swizzle4D,Stack3DImagesWithSameAcqNum,customRename,
         CollapseFolders,AutoCrop, UseGE_0021_104F, PhilipsPrecise: boolean;
         MinReorientMatrix,MaxReorientMatrix,RecursiveFolderDepth: integer;
         BackupDir,NameAppend: string;
  end;

//procedure ReadIni(var lIniName: string; var lPrefs: TPrefs);
procedure SetDefaultPrefs (var lPrefs: TPrefs);
//procedure SaveIni (var lIniName: string; var lPrefs: TPrefs);
procedure CorrectPrefs (var lPrefs: TPrefs); //ensures only usable file types are created
function IniFile(lRead: boolean; lFilename: string; var lPrefs: TPrefs): boolean;

implementation

procedure CorrectPrefs (var lPrefs: TPrefs); //ensures only usable file types are created
begin
  if lPrefs.SingleNIIFile then
     lPrefs.SPM2 := false; //SPM2 only reads .hdr/.img - loses NIfTI information
  if not lPrefs.SingleNIIFile then
     lPrefs.Gzip := false; //nii.gz is OK, but img.gz is not
end;

procedure SetDefaultPrefs (var lPrefs: TPrefs);
begin
  lPrefs.PhilipsPrecise := false;
  lPrefs.UseGE_0021_104F := false;
  lPrefs.CollapseFolders := false;
  lPrefs.AutoCrop := false; //for dcm2nii - reorient and crop 3D nifti input images...
  lPrefs.CustomRename := false;
  lPrefs.createoutputfolder := false;
  lPrefs.Stack3DImagesWithSameAcqNum := false;
  lPrefs.RecursiveUseNameAppend := false;//changes paramstrs.pas recursive search to add top level folder name
  lPrefs.SingleNIIFile := true;
  lPrefs.Gzip := true;
  lPrefs.SPM2 := false;
  lPrefs.Anonymize := true;
  lPrefs.AppendDate := true;
  lPrefs.AppendAcqSeries := true;
  lPrefs.AppendProtocolName := true;
  lPrefs.AppendPatientName := true;
  lPrefs.AppendFilename := true;
  lPrefs.EveryFile:=true;
  lPrefs.FourD := true;
  lPrefs.enablereorient := true;
  lPrefs.ManualNIfTIConv := true;
  lPrefs.AnonymizeSourceDICOM := false;
  lPrefs.Swizzle4D := true;
  lPrefs.RecursiveFolderDepth := 5;
  lPrefs.MinReorientMatrix := 255;
  lPrefs.MaxReorientMatrix := 1023;
  lPrefs.NameAppend := '';
  lPrefs.BackupDir := '';
end;

procedure IniInt(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: integer);
//read or write an integer value to the initialization file
var
	lStr: string;
begin
        if not lRead then begin
           lIniFile.WriteString('INT',lIdent,IntToStr(lValue));
           exit;
        end;
	lStr := lIniFile.ReadString('INT',lIdent, '');
	if length(lStr) > 0 then
		lValue := StrToInt(lStr);
end; //IniInt

procedure IniBool(lRead: boolean; lIniFile: TIniFile; lIdent: string;  var lValue: boolean);
//read or write a boolean value to the initialization file
var
	lStr: string;
begin
        if not lRead then begin
           lIniFile.WriteString('BOOL',lIdent,Bool2Char(lValue));
           exit;
        end;
	lStr := lIniFile.ReadString('BOOL',lIdent, '');
	if length(lStr) > 0 then
		lValue := Char2Bool(lStr[1]);
end; //IniBool

procedure IniStr(lRead: boolean; lIniFile: TIniFile; lIdent: string; var lValue: string);
//read or write a string value to the initialization file
begin
  if not lRead then begin
    lIniFile.WriteString('STR',lIdent,lValue);
    exit;
  end;
	lValue := lIniFile.ReadString('STR',lIdent, '');
end; //IniStr

function IniFile(lRead: boolean; lFilename: string; var lPrefs: TPrefs): boolean;
//Read or write initialization variables to disk
var
  lIniFile: TIniFile;
  //lI: integer;
begin
  result := false;

  if (lRead) and (not Fileexists(lFilename)) then begin
        //FillEmptyMRU(lPrefs);
        exit;
  end;
  lIniFile := TIniFile.Create(lFilename);
IniBool(lRead,lIniFile,'Anonymize',lPrefs.Anonymize);
IniBool(lRead,lIniFile, 'AnonymizeSourceDICOM',lPrefs.AnonymizeSourceDICOM);
IniBool(lRead,lIniFile,'AppendAcqSeries',lPrefs.AppendAcqSeries);
IniBool(lRead,lIniFile,'AppendDate',lPrefs.AppendDate);
IniBool(lRead,lIniFile,'AppendFilename',lPrefs.AppendFilename);
IniBool(lRead,lIniFile,'AppendPatientName',lPrefs.AppendPatientName);
IniBool(lRead,lIniFile,'AppendProtocolName',lPrefs.AppendProtocolName);
IniBool(lRead,lIniFile,'AutoCrop',lPrefs.AutoCrop);
IniBool(lRead,lIniFile,'CollapseFolders',lPrefs.CollapseFolders);
IniBool(lRead,lIniFile,'createoutputfolder',lPrefs.createoutputfolder);
IniBool(lRead,lIniFile,'CustomRename',lPrefs.CustomRename);
IniBool(lRead,lIniFile,'enablereorient',lPrefs.enablereorient);
IniBool(lRead,lIniFile,'EveryFile',lPrefs.EveryFile);
IniBool(lRead,lIniFile,'fourD',lPrefs.fourD);
IniBool(lRead,lIniFile,'Gzip',lPrefs.Gzip);
IniBool(lRead,lIniFile,'ManualNIfTIConv',lPrefs.ManualNIfTIConv);
IniBool(lRead,lIniFile,'PhilipsPrecise',lPrefs.PhilipsPrecise);
IniBool(lRead,lIniFile,'RecursiveUseNameAppend',lPrefs.RecursiveUseNameAppend);
IniBool(lRead,lIniFile,'SingleNIIFile',lPrefs.SingleNIIFile);
IniBool(lRead,lIniFile,'SPM2',lPrefs.SPM2);
IniBool(lRead,lIniFile,'Stack3DImagesWithSameAcqNum',lPrefs.Stack3DImagesWithSameAcqNum);
IniBool(lRead,lIniFile,'Swizzle4D',lPrefs.Swizzle4D);
IniBool(lRead,lIniFile,'UseGE_0021_104F',lPrefs.UseGE_0021_104F);
	IniInt(lRead,lIniFile,'MaxReorientMatrix',lPrefs.MaxReorientMatrix);
	IniInt(lRead,lIniFile,'MinReorientMatrix',lPrefs.MinReorientMatrix);
	IniInt(lRead,lIniFile,'RecursiveFolderDepth',lPrefs.RecursiveFolderDepth);
  lPrefs.BackupDir := lIniFile.ReadString('STR','BackupDir',lPrefs.BackupDir);
//string NameAppend
  lIniFile.Free;
  //lPrefs.PhilipsPrecise := true;
end;

end.



