unit graphx; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  Buttons, StdCtrls, Spin, ComCtrls,metagraph,define_types,text, Menus;

type

  { TGraph4DForm }

  TGraph4DForm = class(TForm)
    Image1: TImage;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    CopyMenu: TMenuItem;
    CloseMenu: TMenuItem;
    FSLBatchMenu: TMenuItem;
    SaveMenu: TMenuItem;
    OpenMenu: TMenuItem;
    MinEdit: TFloatSpinEdit;
    MaxEdit: TFloatSpinEdit;
    HSpeedDrop: TComboBox;
    PSPlot: TSpeedButton;
    PSTextBtn: TSpeedButton;
    SelectDirectoryDialog1: TSelectDirectoryDialog;
    StatusBar1: TStatusBar;
    TrackBar1: TTrackBar;
    TREdit: TFloatSpinEdit;
    FourDBar: TPanel;
    Label1: TLabel;
    SpeedButton1: TSpeedButton;
    RefreshBtn: TSpeedButton;
    procedure CloseMenuClick(Sender: TObject);
    procedure CopyMenuClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FSLBatchMenuClick(Sender: TObject);
    procedure PSPlotClick(Sender: TObject);
    procedure PSTextBtnClick(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure SaveMenuClick(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure Plot4DTrace(lStartSample: integer);
    procedure OpenDataClick(Sender: TObject);
    //function ReadGraf(lFilename: string): boolean;
    function ReadGraf(lFilename: string; lBatch,lTRcritical: boolean): boolean;
    procedure RefreshBtnMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);

  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  Graph4DForm: TGraph4DForm;

implementation

{ TGraph4DForm }

uses
    nifti_img_view, nifti_img,nifti_hdr, nifti_hdr_view,perisettings,periutils,periplot,reslice_fsl;

const
     //kMaxCond = 6;
     kMaxLines = kMaxCond* knMaxOverlay;
     //kClrRA: array [1..kMaxCond] of TColor = (clRed,clBlue,clGreen,clTeal,clAqua,clSilver);
     //kPenStyleRA: array[1..kVOIOverlayNum] of TPenStyle = (psDot,psDot,psDash,psDashDot,psDashDotDot);//abba
     //kPenStyleRA: array[1..kVOIOverlayNum] of TPenStyle = (psSolid,psDot,psDash,psDashDot,psDashDotDot);


var
   g4DHdr: TMRIcroHdr;
   g4Ddata: T4DTrace;
   
   function ROIoverlayNum(lROI: integer): integer;
var
   lR,lN: integer;
begin
     result := 0;
     lN := 0;
     for lR := (kBGOverlayNum+1) to knMaxOverlay do begin
              //fx(lR,gMRIcroOverlay[lR].ScrnBufferItems);
              if gMRIcroOverlay[lR].ScrnBufferItems > 0 then begin
                 inc(lN);
                 if lROI = lN then begin
                     result := lR;
                     exit;
                 end;
              end; //if ROI has items
     end;
end;
   
function ROIoverlayNameShort(lROI: integer): string;
begin
     if ROIoverlayNum(lROI) = 0 then
        result := inttostr(ImgForm.XViewEdit.value)+'x'+inttostr(ImgForm.YViewEdit.value)+'x'+inttostr(ImgForm.ZViewEdit.value)
     else
         result := parsefilename(extractfilename(gMRIcroOverlay[ROIoverlayNum(lROI)].HdrFileName));
end;

function ROIoverlayNameLong(lROI: integer): string;
begin
     if ROIoverlayNum(lROI) = 0 then
        result := inttostr(ImgForm.XViewEdit.value)+'x'+inttostr(ImgForm.YViewEdit.value)+'x'+inttostr(ImgForm.ZViewEdit.value)
     else
         result := gMRIcroOverlay[ROIoverlayNum(lROI)].HdrFileName;
end;
   
function ROImean (var l4DHdr: TMRIcroHdr; lROInum,lVol,lVolSz: integer): single;
var
   lVox,lCount,lVolOffset: integer;
   lSum: double;
   l16Buf : SmallIntP;
   l32Buf : SingleP;
begin
     lVolOffset := (lVol-1)*lVolSz;
     //next - compute sum of signal - unrolled loops for each datatype
     lCount := 0;
     lSum := 0;
     if (l4DHdr.ImgBufferBPP  = 4) then begin
	l32Buf := SingleP(l4DHdr.ImgBuffer );
        for lVox := 1 to lVolSz do begin
            if gMRIcroOverlay[lROInum].ScrnBuffer^[lVox] > 0 then begin
               inc(lCount);
               lSum := lSum + l32Buf^[lVox+lVolOffset];
            end; //part of ROI
        end; //for each vox
     end else if (l4DHdr.ImgBufferBPP  = 2) then begin
        l16Buf := SmallIntP(l4DHdr.ImgBuffer );
        for lVox := 1 to lVolSz do begin
            if gMRIcroOverlay[lROInum].ScrnBuffer^[lVox] > 0 then begin
               inc(lCount);
               lSum := lSum + l16Buf^[lVox+lVolOffset];
            end; //part of ROI
        end; //for each vox
     end else if l4DHdr.ImgBufferBPP  = 1 then begin
        for lVox := 1 to lVolSz do begin
            if gMRIcroOverlay[lROInum].ScrnBuffer^[lVox] > 0 then begin
               inc(lCount);
               lSum := lSum + l4DHdr.ImgBuffer^[lVox+lVolOffset];
            end; //part of ROI
        end; //for each vox
     end else
         showmessage('Serious error: unknown data size!');
     //now compute mean signal
     if lCount > 0 then
        result := lSum / lCount
     else
         result := 0;
end;

procedure TextToTrace (var l4DTrace: T4DTrace);
var
   lStr: string;
   lCond, lnCond,lE: integer;
begin
     lncond := 0;
     for lCond := 1 to kMaxCond do
         if l4DTrace.Lines[lCond].Events > 0 then
            inc(lnCond);
     if lncond = 0 then
     exit;
     TextForm.Memo1.Lines.Clear;//prepare to report results
     for lCond := 1 to kMaxCond do begin
         if l4DTrace.Lines[lCond].Events > 0 then begin
            lStr := l4DTrace.Lines[lCond].ELabel;
            for lE := 1 to l4DTrace.Lines[lCond].Events do
                lStr := lStr + ','+ realtostr(l4DTrace.Lines[lCond].EventRA^[lE],4) ;
            TextForm.Memo1.lines.add(lStr);

         end;
     end;
     TextForm.show;
end;



function TGraph4DForm.ReadGraf(lFilename: string; lBatch,lTRcritical: boolean): boolean;
//function TGraph4DForm.ReadGraf(lFilename: string): boolean;
label
     666;
var
   lnVol: integer;
   lReslice : boolean;
begin
     ImgForm.CloseImagesClick(nil);
     Close4DTrace(g4Ddata,true);
     FreeImgMemory(g4DHdr);
     result := false;
     if not fileexists(lFilename) then exit;
     Graph4DForm.Caption := 'Viewing: '+lFilename;
     lReslice := gBGImg.ResliceOnLoad;
     gBGImg.ResliceOnLoad := false;
     gBGImg.Prompt4DVolume := false;
    ImgForm.OpenAndDisplayImg(lFilename,True);
    lnVol := gMRIcroOverlay[kBGOverlayNum].NIFTIhdr.dim[4];
    if (lnVol < 2) then begin
           showmessage('You need to open a 4D image.');
           goto 666;
    end;
    if not HdrForm.OpenAndDisplayHdr(lFilename,g4DHdr) then goto 666;
    if not OpenImg(gBGImg,g4DHdr,false,false,false,false,true {4D!}) then goto 666;



(*     //if not lBatch then begin  //12/2007
        if not HdrForm.OpenAndDisplayHdr(lFilename,g4DHdr) then goto 666;


        ImgForm.OpenAndDisplayImg(lFilename,True);
        lnVol := gMRIcroOverlay[kBGOverlayNum].NIFTIhdr.dim[4];
        if (lnVol < 2) then begin
           showmessage('You need to open a 4D image.');
           goto 666;
        end;
    //end;
    if not OpenImg(gBGImg,g4DHdr,false,false,false,false,true {4D!}) then goto 666; *)
    TrackBar1.Max := lnVol;
    if g4DHdr.NIFTIhdr.PixDim[4] = 0 then begin
      beep;
      ImgForm.StatusLabel.caption := 'Assuming TR = '+floattostr(TREdit.value);
    end else
        TREdit.value := g4DHdr.NIFTIhdr.PixDim[4];//TR
    if (TREdit.value = 0) and (lTRcritical) then
       showmessage('Please set the TR value [in seconds]');
    //RefreshBtnClick(nil);
     result := true;
666:
     gBGImg.ResliceOnLoad := lReslice;
     gBGImg.Prompt4DVolume := true;
end;
(*label
     666;
var
   lnVol: integer;
   lReslice : boolean;
begin
     ImgForm.CloseImagesClick(nil);
     Close4DTrace(g4Ddata,true);
     FreeImgMemory(g4DHdr);
     result := false;
     if not fileexists(lFilename) then exit;
     Graph4DForm.Caption := 'Viewing: '+lFilename;
     lReslice := gBGImg.ResliceOnLoad;
     gBGImg.ResliceOnLoad := false;
     ImgForm.OpenAndDisplayImg(lFilename,True);
     lnVol := gMRIcroOverlay[kBGOverlayNum].NIFTIhdr.dim[4];
     if (lnVol < 2) then begin
        showmessage('You need to open a 4D image.');
        goto 666;
     end;
     if not HdrForm.OpenAndDisplayHdr(lFilename,g4DHdr) then goto 666;
    if not OpenImg(gBGImg,g4DHdr,false,false,false,false,true {4D!}) then goto 666;
    TrackBar1.Max := lnVol;
    TREdit.value := g4DHdr.NIFTIhdr.PixDim[4];//TR
    if TREdit.value = 0 then
       showmessage('Please set the TR value [in seconds]');
    //RefreshBtnClick(nil);
     result := true;
666:
     gBGImg.ResliceOnLoad := lReslice;
end;*)

procedure TGraph4DForm.RefreshBtnMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
         ConvertToTrace(g4DHdr,g4Ddata,ImgForm.XViewEdit.value,ImgForm.YViewEdit.value,ImgForm.ZViewEdit.value);
	 if (ssAlt in Shift) then
            TextToTrace (g4Ddata);
         Plot4DTrace(TrackBar1.position);
end;

procedure TGraph4DForm.Plot4DTrace(lStartSample: integer);
begin
     g4Ddata.HorzWidPerBin := TREdit.value;
     CorePlot4DTrace(g4Ddata,Image1,lStartSample,HSpeedDrop.ItemIndex,-1,TREdit.value,MinEdit.value,MaxEdit.value,false);
end;


{$DEFINE notTest4D}
procedure TGraph4DForm.OpenDataClick(Sender: TObject);
var
   lI,lCnt: integer;
   lStr: string;
begin
     Close4DTrace(g4Ddata,true);
     FreeImgMemory(g4DHdr);
{$IFDEF Test4D}
     if not ReadGraf('C:\fatigue\TD\td.nii.gz',false,true) then exit;
     ReadCond('C:\fatigue\TD\a.txt',g4Ddata,1);
     ReadCond('C:\fatigue\TD\b.txt',g4Ddata,2);
     PSPlotClick(nil);
     exit;

     {if not ReadGraf('C:\fatigue\a60z.hdr',false,true) then exit;
     ReadCond('C:\fatigue\a1.txt',g4Ddata,1);
     ReadCond('C:\fatigue\a2.txt',g4Ddata,2);
     PSPlotClick(nil);  }
     exit;
{$ENDIF}
     if not OpenDialogExecute(kImgFilter,'Select 4D image',false) then exit;
     if not ReadGraf(HdrForm.OpenHdrDlg.Filename,false,true) then exit;
     ImgForm.XViewEdit.value := gMRIcroOverlay[kBGOverlayNum].NIFTIhdr.dim[1] div 2;
     ImgForm.YViewEdit.value := gMRIcroOverlay[kBGOverlayNum].NIFTIhdr.dim[2] div 2;
     ImgForm.ZViewEdit.value := gMRIcroOverlay[kBGOverlayNum].NIFTIhdr.dim[3] div 2;
     if OpenDialogExecute(kTxtFilter,'Select 3-column event onset time files [optional]',true) then begin
        if HdrForm.OpenHdrDlg.Files.Count > 0 then begin
           lCnt := HdrForm.OpenHdrDlg.Files.Count;
           if lCnt > kMaxCond then begin
               showmessage('Can only load '+inttostr(lCnt)+'conditions');
               lCnt := kMaxCond;
           end;
           for lI := 1 to lCnt do
               ReadCond(HdrForm.OpenHdrDlg.Files[lI-1],g4Ddata,lI);
        end;//if count > 1
     end; //if opendialog
     if OpenDialogExecute(kImgPlusVOIFilter,'Select regions of interest',true) then begin
        if HdrForm.OpenHdrDlg.Files.Count > 0 then begin
           lCnt := HdrForm.OpenHdrDlg.Files.Count;
           //Apr07
           if lCnt > (knMaxOverlay-2) then begin
               showmessage('Can only load '+inttostr(knMaxOverlay-2)+'conditions');
               lCnt := knMaxOverlay;
           end;
           for lI := 1 to lCnt do begin
               lStr := HdrForm.OpenHdrDlg.Files[lI-1];
               ImgForm.OverlayOpenCore(lStr,lI+kBGOverlayNum);
           end;
        end;//if count > 1
     end; //if opendialog
          RefreshBtnMouseDown(nil,mbleft,[],1,1);
end;

procedure TGraph4DForm.FormCreate(Sender: TObject);
begin
     Create4DTrace(g4Ddata);
     Graph4DForm.DoubleBuffered := true;
     HSpeedDrop.ItemIndex := 0;
     InitImgMemory(g4DHdr);

end;

procedure TGraph4DForm.FormResize(Sender: TObject);
begin
    if not Graph4DForm.visible then exit;
    GraphResize(Image1);
     Plot4DTrace(TrackBar1.position);
end;

procedure TGraph4DForm.FormShow(Sender: TObject);
begin
  //nothing
end;


procedure TGraph4DForm.PSPlotClick(Sender: TObject);
var
   lPSPlot: TPSPlot;
begin
     if NCond ( g4Ddata) < 1 then begin
        RefreshBtnMouseDown(nil,mbleft,[],1,1);
        exit;
     end;
     lPSPlot.TRSec := TREdit.value;
     if not PSForm.GetPeriSettings(lPSPlot) then
        exit;
     lPSPlot.TextOutput := false;
    lPSPlot.GraphOutput := true;
    lPSPlot.batch := false;
     CreatePeristimulusPlot (g4DHdr,g4Ddata, lPSPlot);
end;

procedure TGraph4DForm.PSTextBtnClick(Sender: TObject);
var lPSPlot: TPSPlot;
begin
     if NCond ( g4Ddata) < 1 then begin
        RefreshBtnMouseDown(nil,mbleft,[],1,1);
        TextForm.Memo1.Lines.Clear;//prepare to report results
        TextToTrace (g4Ddata);
        TextForm.show;
        exit;
     end;
     lPSPlot.TRSec := TREdit.value;
     if not PSForm.GetPeriSettings(lPSPlot) then
        exit;
     lPSPlot.TextOutput := true;
     lPSPlot.GraphOutput := true;
     lPSPlot.batch := false;
     CreatePeristimulusPlot (g4DHdr,g4Ddata, lPSPlot);
     end;

procedure TGraph4DForm.RefreshBtnClick(Sender: TObject);
begin

end;

procedure TGraph4DForm.SaveMenuClick(Sender: TObject);
begin
  	SaveImgAsPNGBMP (Image1);
end;

procedure TGraph4DForm.TrackBar1Change(Sender: TObject);
begin
          Trackbar1.visible := (HSpeedDrop.ItemIndex > 0);
     Plot4DTrace(TrackBar1.position);
end;

procedure TGraph4DForm.FormClose(Sender: TObject; var CloseAction: TCloseAction
  );
begin
//not for lazarus?
       Close4DTrace(g4Ddata,true);
     FreeImgMemory(g4DHdr);
end;

procedure TGraph4DForm.CopyMenuClick(Sender: TObject);
begin
        {$IFDEF zxDarwin}
        Showmessage('Copy not yet supported with OSX: use File/Save');
        exit;
        {$ENDIF}


	 if (Image1.Picture.Graphic = nil) then begin //1420z
        Showmessage('You need to generate an image before you can copy it to the clipboard.');
        exit;
     end;
        Image1.Picture.Bitmap.SaveToClipboardFormat(2);
end;

procedure TGraph4DForm.CloseMenuClick(Sender: TObject);
begin
       Graph4DForm.Close;
end;

function ResliceFSLVOIs(var lFeatDirs,lVOI: TStringList): boolean;
//uses reslice
var
   lDir,lV: integer;
   lMatName,lFuncName,lReslicedVOIName:string;
begin
    result := false;
    if lFeatDirs.count < 1 then exit;
    if lVOI.count < 1 then exit;
    for lDir := 1 to (lFeatDirs.Count) do begin
        lMatName := FSLMatName (lFeatDirs[lDir-1]);
        lFuncName := FSLFuncName (lFeatDirs[lDir-1]);
        for lV := 1 to lVOI.Count do begin
            lReslicedVOIName := FSLReslicedVOIName (lFeatDirs[lDir-1], lVOI[lV-1]);
            //showmessage(lReslicedVOIName);
            if not ResliceImg (lFuncName,lVOI[lV-1],lMatName,lReslicedVOIName) then begin
               //fx(999);
               exit;
            end;
        end;//for each VOI
    end;//for each Dir
    result := true;
end;

{$DEFINE notTEST}

function FindFEATFolders (var lFeatDirs:TStringList): boolean;
var
   lDir,lFeatPath: string;
   lSearchRec: TSearchRec;
begin
     result := false;
     {$IFDEF TEST}
     lDir := 'C:\cygwin\home\express';
     {$ELSE}
     {$IFDEF FPC}
     if not Graph4DForm.SelectDirectoryDialog1.execute then exit;
     lDir := Graph4DForm.SelectDirectoryDialog1.FileName;
     //showmessage(lDir);
     
     
     {$ELSE}
     lDir := SelectDirectory('Choose root folder that contains .feat folders', BIF_RETURNONLYFSDIRS);
     {$ENDIF}
     {$ENDIF}
     if lDir = '' then exit;
     lFeatDirs := TStringList.Create;
     if FindFirst(lDir+pathdelim+'*'+'.feat', faAnyFile, lSearchRec) = 0 then begin
	    repeat
          if (faDirectory and lSearchRec.attr) = faDirectory then begin
             lFeatPath := lDir+pathdelim+lSearchRec.Name;
             if Fileexists(FSLMatName(lFeatPath)) and Fileexists(FSLFuncName(lFeatPath)) then
		        lFeatDirs.Add(lFeatPath)
             else
                 Showmessage('Can not find '+FSLMatName(lFeatPath) +' or '+FSLFuncName(lFeatPath) );
          end;
        until (FindNext(lSearchRec) <> 0);
     end;
     FindClose(lSearchRec);
     if lFeatDirs.Count < 1  then begin
        Showmessage('Unable to find any feat dirs in path '+lDir+pathdelim+'*'+'.FEAT');
        lFeatDirs.free;
        exit;
     end;
     result := true;
end;



procedure TGraph4DForm.FSLBatchMenuClick(Sender: TObject);
label
     111;
var
   lStr: string;
   lFeatDirs,lVectors,lVOI: TStringList;
   //lTRSec,lBinWidthSec: single;
   lI,lImg{,lnNegBins,lnPosBins}: integer;
   lUseFSLEVs: Boolean;
   lPSPlot: TPSPlot;
   {lSliceTime,lSavePSVol,lTextOutput,lBaselineCorrect,lPctSignal,
   lRemoveRegressorVariability,lTemporalDeriv,lUseFSLEVs,lPlotModel: boolean; }
begin
     ImgForm.CloseImagesClick(nil);
     Close4DTrace(g4Ddata,true);
     FreeImgMemory(g4DHdr);
     if not FindFEATFolders (lFeatDirs) then
      exit;
     lVectors := TStringList.Create;//empty
     lVOI := TStringList.Create;
     {$IFDEF TEST}
     lVectors.Add('C:\cygwin\home\express\20070420_132327fMRIcontin30x30x36s004a001.feat\custom_timing_files\ev1.txt');
     lVectors.Add('C:\cygwin\home\express\20070420_132327fMRIcontin30x30x36s004a001.feat\custom_timing_files\ev2.txt');
     lVectors.Add('C:\cygwin\home\express\20070420_132327fMRIcontin30x30x36s004a001.feat\custom_timing_files\ev3.txt');
     lVOI.Add('C:\fatigue\v1.nii.gz');
     {$ELSE}
     FSLEVNames (lFeatDirs[0], lVectors);
     lUseFSLEVs := false;
     if lVectors.count > 0 then
        lUseFSLEVs := OKMsg('Use event vectors from the .FEAT'+pathdelim+'custom_timing_files folder?'); //shows dialog with OK/Cancel returns true if user presses OK
     if not lUseFSLEVs then begin
        lVectors.clear;
        if  OpenDialogExecute(kTxtFilter,'Select 3-column event onset time files',true) then begin
         if HdrForm.OpenHdrDlg.Files.Count > kMaxCond then begin
               showmessage('Can only load '+inttostr(kMaxCond)+'conditions');
               goto 111;
         end;
         lVectors.AddStrings(HdrForm.OpenHdrDlg.Files);
        end;
     end; //manually select EVs

     if not OpenDialogExecute(kImgPlusVOIFilter,'Select volume[s] of interest [2mm MNI space]',true) then
        goto 111;
     if HdrForm.OpenHdrDlg.Files.Count > (knMaxOverlay-2) then begin
        showmessage('Can only load '+inttostr(knMaxOverlay-2)+'conditions');
        goto 111;
     end;
     lVOI.AddStrings(HdrForm.OpenHdrDlg.Files);
     {$ENDIF}
     if not ResliceFSLVOIs(lFeatDirs,lVOI) then begin
        showmessage('Unable to reslice VOIs!');
        goto 111;
     end;
     lPSPlot.TextOutput := true;
     lPSPlot.GraphOutput := false;
     TextForm.Memo1.Lines.Clear;//prepare to report results


     if not ReadGraf(FSLFuncName (lFeatDirs[0]),false, (lVectors.count > 0) ) then
        goto 111; //read first dataset to set TR!
     //la1 := (FreeRAM);
     //get plot settings....
     lPSPlot.TRSec := TREdit.value;
     if lVectors.count > 0 then
        if not PSForm.GetPeriSettings(lPSPlot) then
           goto 111;

     //Textform.memo1.lines.add('RAM=,'+inttostr(FreeRAM)+',1');//rascal
     for lImg := 1 to lFeatDirs.Count do begin
         if lImg > 1 then begin//we have already read 1st img
            Refresh;
            Application.processmessages;
            Close4DTrace(g4Ddata,true);
            ImgForm.CloseImagesClick(nil);
            FreeImgMemory(g4DHdr);
            if not ReadGraf(FSLFuncName (lFeatDirs[lImg-1]),true,(lVectors.count > 0)) then
               goto 111; //read first dataset to set TR!
            if lUseFSLEVs then
               FSLEVNames (lFeatDirs[lImg-1], lVectors)
     //Textform.memo1.lines.add('RAM=,'+inttostr(FreeRAM)+',1');//rascal
         end; //all except 1st image
         if lVectors.count > 0 then begin
            for lI := 1 to lVectors.count do
                ReadCond(lVectors[lI-1],g4Ddata,lI);
         end;//vectors > 0
         if lVOI.count > 0 then begin
            for lI := 1 to lVOI.count do begin
               lStr := FSLReslicedVOIName (lFeatDirs[lImg-1], lVOI[lI-1]);
               ImgForm.OverlayOpenCore(lStr,lI+kBGOverlayNum);
            end;//for each VOI
         end; //VOI > 0
         if lVectors.Count > 0 then begin
          if lImg = lFeatDirs.Count then
             lPSPlot.GraphOutput := true;
          CreatePeristimulusPlot (g4DHdr,g4Ddata, lPSPlot)
         end else begin
              ConvertToTrace(g4DHdr,g4Ddata,ImgForm.XViewEdit.value,ImgForm.YViewEdit.value,ImgForm.ZViewEdit.value);
             TextToTrace (g4Ddata);
             RegressTrace(g4Ddata);
         end;
     end;
          TextForm.show;
     111:
     lVOI.Free;
     lVectors.Free;
     lFeatDirs.free;
end;


initialization
  {$I graphx.lrs}

end.

